import 'dart:async';
import 'dart:typed_data';

import 'package:flutter/material.dart';
import 'package:photo/src/delegate/badge_delegate.dart';
import 'package:photo/src/delegate/checkbox_builder_delegate.dart';
import 'package:photo/src/delegate/loading_delegate.dart';
import 'package:photo/src/engine/lru_cache.dart';
import 'package:photo/src/engine/throttle.dart';
import 'package:photo/src/entity/options.dart';
import 'package:photo/src/provider/asset_provider.dart';
import 'package:photo/src/provider/config_provider.dart';
import 'package:photo/src/provider/gallery_list_provider.dart';
import 'package:photo/src/provider/i18n_provider.dart';
import 'package:photo/src/provider/selected_provider.dart';
import 'package:photo/src/ui/dialog/change_gallery_dialog.dart';
import 'package:photo/src/ui/page/main/empty_widget.dart';
import 'package:photo/src/ui/page/photo_preview_page.dart';
import 'package:photo_manager/photo_manager.dart';
import 'package:provider/provider.dart';

part './main/bottom_widget.dart';
part './main/image_item.dart';

class PhotoMainPage extends StatefulWidget {
  final ValueChanged<PickedEntity> onClose;
  final Options options;
  final List<AssetPathEntity> photoList;

  const PhotoMainPage({
    Key key,
    this.onClose,
    this.options,
    this.photoList,
  }) : super(key: key);

  @override
  _PhotoMainPageState createState() => _PhotoMainPageState();
}

class _PhotoMainPageState extends State<PhotoMainPage>
    with SelectedProvider, GalleryListProvider, WidgetsBindingObserver {
  PhotoProvider get config =>
      Provider.of<PhotoProvider>(context, listen: false);
  Options get options => widget.options;
  I18nProvider get i18nProvider => config.provider;
  AssetProvider get assetProvider => config.assetProvider;

  List<AssetEntity> get list => assetProvider.data;

  bool _isInit = false;

  String get currentGalleryName {
    if (assetProvider.current?.isAll == true) {
      return i18nProvider.getAllGalleryText(options);
    }
    return assetProvider.current?.name ?? "Select Folder";
  }

  GlobalKey scaffoldKey;
  ScrollController scrollController;

  bool isPushed = false;

  bool get useAlbum => widget.photoList == null || widget.photoList.isEmpty;

  Throttle _changeThrottle;

  OverlayEntry overlayEntry;

  @override
  void initState() {
    super.initState();
    scaffoldKey = GlobalKey();
    scrollController = ScrollController();
    _changeThrottle = Throttle(onCall: _onAssetChange);
    PhotoManager.addChangeCallback(_changeThrottle.call);
    PhotoManager.startChangeNotify();
    WidgetsBinding.instance.addPostFrameCallback((timeStamp) async {
      if (!_isInit) {
        final pickedList = config.pickedAssetList ?? [];
        addPickedAsset(pickedList.toList());
        _refreshList();
      }
    });
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
  }

  @override
  void dispose() {
    PhotoManager.removeChangeCallback(_changeThrottle.call);
    PhotoManager.stopChangeNotify();
    _changeThrottle.dispose();
    scaffoldKey = null;
    super.dispose();
  }

  void _closeGallerySelectDialog() {
    overlayEntry.remove();
    overlayEntry = null;
    setState(() {});
  }

  void _showGallerySelectDialog() {
    if (overlayEntry != null) {
      _closeGallerySelectDialog();
      return;
    }
    overlayEntry = OverlayEntry(builder: (context) {
      return Positioned(
        top: kToolbarHeight + MediaQuery.of(context).padding.top,
        right: 0,
        left: 0,
        bottom: 0,
        child: WillPopScope(
          onWillPop: () async {
            _closeGallerySelectDialog();
            return false;
          },
          child: Stack(
            children: <Widget>[
              GestureDetector(
                onTap: _closeGallerySelectDialog,
                child: Container(
                  height: double.infinity,
                  color: Color(0x66000000),
                ),
              ),
              Material(
                color: options.theme.surfaceColor,
                child: Container(
                  height: MediaQuery.of(context).size.height * 0.5 -
                      MediaQuery.of(context).padding.top,
                  child: ChangeGalleryDialog(
                      galleryList: galleryPathList,
                      i18n: i18nProvider,
                      options: options,
                      current: assetProvider.current,
                      onGalleryChange: (AssetPathEntity result) {
                        _onGalleryChange?.call(result);
                      }),
                ),
              ),
            ],
          ),
        ),
      );
    });
    Overlay.of(context).insert(overlayEntry);
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return DefaultTextStyle(
      style: options.theme.textStyle,
      child: Scaffold(
        appBar: AppBar(
          titleSpacing: 0.0,
          backgroundColor: options.theme.surfaceColor,
          automaticallyImplyLeading: false,
          centerTitle: true,
          title: Row(
              mainAxisAlignment: MainAxisAlignment.start,
              crossAxisAlignment: CrossAxisAlignment.center,
              children: [
                GestureDetector(
                  behavior: HitTestBehavior.opaque,
                  onTap: _cancel,
                  child: Container(
                    height: kToolbarHeight,
                    padding: kTabLabelPadding,
                    alignment: Alignment.center,
                    child: Text(i18nProvider.getCancelText(options), style: options.theme.textStyle),
                  ),
                ),
                Expanded(
                  child: FlatButton(
                    padding: EdgeInsets.zero,
                    onPressed: _showGallerySelectDialog,
                    splashColor: Colors.transparent,
                    highlightColor: Colors.transparent,
                    child: Container(
                        alignment: Alignment.center,
                        height: 44.0,
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.center,
                          children: [
                            Text(
                              currentGalleryName,
                              style: options.theme.textStyle.apply(
                                fontSizeDelta: 2.0,
                                fontWeightDelta: 1,
                              ),
                            ),
                            Container(
                              width: 18,
                              height: 18,
                              margin: EdgeInsets.only(left: 8),
                              decoration: ShapeDecoration(
                                  shape: CircleBorder(
                                side: BorderSide(
                                    color: options.theme.onSurfaceColor,
                                    width: 1.5),
                              )),
                              child: AnimatedSwitcher(
                                transitionBuilder: (child, animation) {
                                  final Tween<double> tween = overlayEntry != null
                                      ? Tween(begin: -0.25, end: 0.25)
                                      : Tween(begin: 0.25, end: 0.75);
                                  //var tween = Tween(begin: -0.5, end: 0.0);
                                  return RotationTransition(
                                      child: child,
                                      turns: tween.animate(animation));
                                },
                                duration: Duration(milliseconds: 200),
                                reverseDuration: Duration(milliseconds: 0),
                                child: Icon(Icons.chevron_left,
                                    key: ValueKey(overlayEntry == null ? 0 : 1),
                                    size: 14,
                                    color: options.theme.onSurfaceColor),
                              ),
                            ),
                          ],
                        )),
                  ),
                ),
                Container(
                  height: kToolbarHeight,
                  padding: kTabLabelPadding,
                  alignment: Alignment.center,
                  child: Text(i18nProvider.getCancelText(options), style: options.theme.textStyle.copyWith(
                    color: Colors.transparent,
                  )),
                ),
              ],
          ),
        ),
        body: _buildBody(),
        bottomNavigationBar: options.maxSelected > 1 ? _BottomWidget(
          key: scaffoldKey,
          provider: i18nProvider,
          options: options,
          onTapPreview: selectedList.isEmpty ? null : _onTapPreview,
          onTapSure: selectedList.isEmpty ? null : sure,
          selectedProvider: this,
          galleryListProvider: this,
        ) : null,
      ),
    );
  }

  void _cancel() {
    if (overlayEntry != null) {
      _closeGallerySelectDialog();
      return;
    }
    selectedList.clear();
    widget.onClose(PickedEntity(
      asset: selectedList,
      isFullImage: isFullImage,
    ));
  }

  @override
  bool isUpperLimit() {
    var result = selectedCount == options.maxSelected;
    if (result) _showTip(i18nProvider.getMaxTipText(options));
    return result;
  }

  void sure() {
    widget.onClose?.call(PickedEntity(
      asset: selectedList,
      isFullImage: isFullImage,
    ));
  }

  void _showTip(String msg) {
    if (isPushed) {
      return;
    }
    Scaffold.of(scaffoldKey.currentContext).showSnackBar(
      SnackBar(
        content: Text(
          msg,
          style: options.theme.textStyle
              .copyWith(color: options.theme.onPrimaryColor),
        ),
        duration: Duration(milliseconds: 1500),
        backgroundColor: options.theme.primaryColor,
      ),
    );
  }

  void _refreshList() async {
    await Future.delayed(Duration.zero);
    print(useAlbum);
    if (!useAlbum) {
      _refreshListFromWidget();
      return;
    }

    _refreshListFromGallery();
  }

  Future<void> _refreshListFromWidget() async {
    _onRefreshAssetPathList(widget.photoList);
  }

  Future<void> _refreshListFromGallery() async {
    List<AssetPathEntity> pathList;
    switch (options.pickType) {
      case PickType.onlyImage:
        pathList = await PhotoManager.getAssetPathList(type: RequestType.image);
        break;
      case PickType.onlyVideo:
        pathList = await PhotoManager.getAssetPathList(type: RequestType.video);
        break;
      default:
        pathList = await PhotoManager.getAssetPathList(
            type: RequestType.image | RequestType.video);
    }

    _onRefreshAssetPathList(pathList);
  }

  Future<void> _onRefreshAssetPathList(List<AssetPathEntity> pathList) async {
    if (pathList == null) {
      return;
    }

    options.sortDelegate.sort(pathList);

    galleryPathList.clear();
    galleryPathList.addAll(pathList);

    if (pathList.isNotEmpty) {
      assetProvider.current = pathList[0];
      await assetProvider.loadMore();
    }

    for (var path in pathList) {
      if (path.isAll) {
        path.name = i18nProvider.getAllGalleryText(options);
      }
    }

    setState(() {
      _isInit = true;
    });
  }

  Widget _buildBody() {
    if (!_isInit) {
      return _buildLoading();
    }

    final noMore = assetProvider.noMore;

    final count = assetProvider.count + (noMore ? 0 : 1);

    return Container(
      color: options.theme.backgroundColor,
      padding: EdgeInsets.all(5),
      child: assetProvider.count == 0
          ? DefaultEmptyDelegate().buildEmpty(context, options)
          : GridView.builder(
              controller: scrollController,
              gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
                crossAxisCount: options.rowCount,
                childAspectRatio: options.itemRadio,
                crossAxisSpacing: options.padding,
                mainAxisSpacing: options.padding,
              ),
              itemBuilder: _buildItem,
              itemCount: count,
            ),
    );
  }

  Widget _buildItem(BuildContext context, int index) {
    final noMore = assetProvider.noMore;
    if (!noMore && index == assetProvider.count) {
      _loadMore();
      return _buildLoading();
    }

    var data = list[index];
    return RepaintBoundary(
      child: GestureDetector(
        onTap: () => _onItemClick(data, index),
        child: Stack(
          children: <Widget>[
            ImageItem(
              entity: data,
              themeColor: options.theme.primaryColor,
              size: options.thumbSize,
              loadingDelegate: options.loadingDelegate,
              badgeDelegate: options.badgeDelegate,
            ),
            if (options.maxSelected > 1)
            _buildMask(containsEntity(data)),
            if (options.maxSelected > 1)
            _buildSelected(data),
          ],
        ),
      ),
    );
  }

  _loadMore() async {
    await assetProvider.loadMore();
    setState(() {});
  }

  _buildMask(bool showMask) {
    return IgnorePointer(
      child: AnimatedContainer(
        color: showMask ? Colors.black.withOpacity(0.5) : Colors.transparent,
        duration: Duration(milliseconds: 300),
      ),
    );
  }

  Widget _buildSelected(AssetEntity entity) {
    var currentSelected = containsEntity(entity);
    return Positioned(
      right: 0.0,
      width: 36.0,
      height: 36.0,
      child: GestureDetector(
        onTap: () {
          changeCheck(!currentSelected, entity);
        },
        behavior: HitTestBehavior.translucent,
        child: _buildText(entity),
      ),
    );
  }

  Widget _buildText(AssetEntity entity) {
    var isSelected = containsEntity(entity);
    Widget child;
    BoxDecoration decoration = BoxDecoration(
      borderRadius: BorderRadius.circular(1.0),
    );
    if (isSelected) {
      child = Text(
        (indexOfSelected(entity) + 1).toString(),
        textAlign: TextAlign.center,
        style: options.theme.textStyle.apply(
          fontSizeDelta: -4,
          color: options.theme.onPrimaryColor,
        ),
      );
      decoration = decoration.copyWith(color: options.theme.primaryColor);
    } else {
      decoration = decoration.copyWith(
        border: Border.all(
          color: options.theme.onPrimaryColor.withAlpha(200),
        ),
      );
    }
    return Padding(
      padding: const EdgeInsets.all(8.0),
      child: AnimatedContainer(
        duration: Duration(milliseconds: 300),
        decoration: decoration,
        alignment: Alignment.center,
        child: child,
      ),
    );
  }

  void changeCheck(bool value, AssetEntity entity) {
    if (value) {
      addSelectEntity(entity);
    } else {
      removeSelectEntity(entity);
    }
    setState(() {});
  }

  void _onGalleryChange(AssetPathEntity assetPathEntity) async {
    _closeGallerySelectDialog();
    if (assetPathEntity != assetProvider.current) {
      assetProvider.current = assetPathEntity;
      await assetProvider.loadMore();
      setState(() {});
    }
  }

  void _onItemClick(AssetEntity data, int index) {
    if (options.maxSelected == 1) {
      widget.onClose?.call(PickedEntity(
        asset: [data],
        isFullImage: true,
      ));
      return;
    }
    var result = PhotoPreviewResult();
    isPushed = true;
    Navigator.of(context).push(
      MaterialPageRoute(
        builder: (ctx) {
          return ChangeNotifierProvider.value(
            value: config,
            child: PhotoPreviewPage(
              selectedProvider: this,
              list: List.of(list),
              initIndex: index,
              changeProviderOnCheckChange: true,
              result: result,
              isPreview: false,
              assetProvider: assetProvider,
            ),
          );
        },
      ),
    ).then((v) {
      if (handlePreviewResult(v)) {
        Navigator.pop(context, v);
        return;
      }
      isPushed = false;
      setState(() {});
    });
  }

  void _onTapPreview() async {
    var result = PhotoPreviewResult();
    isPushed = true;
    var v = await Navigator.of(context).push(
      MaterialPageRoute(
        builder: (ctx) => ChangeNotifierProvider.value(
          value: config,
          child: PhotoPreviewPage(
            selectedProvider: this,
            list: List.of(selectedList),
            changeProviderOnCheckChange: false,
            result: result,
            isPreview: true,
            assetProvider: assetProvider,
          ),
        ),
      ),
    );
    if (handlePreviewResult(v)) {
      // print(v);
      Navigator.pop(context, v);
      return;
    }
    isPushed = false;
    compareAndRemoveEntities(result.previewSelectedList);
    setState(() {});
  }

  bool handlePreviewResult(PickedEntity v) {
    if (v == null) {
      return false;
    }
    if (v is PickedEntity) {
      return true;
    }
    return false;
  }

  Widget _buildLoading() {
    return Center(
      child: Column(
        children: <Widget>[
          Container(
            width: 40.0,
            height: 40.0,
            padding: const EdgeInsets.all(5.0),
            child: CircularProgressIndicator(
              valueColor: AlwaysStoppedAnimation(options.theme.primaryColor),
            ),
          ),
          Padding(
            padding: const EdgeInsets.all(8.0),
            child: Text(
              i18nProvider.loadingText(),
              style: const TextStyle(
                fontSize: 12.0,
              ),
            ),
          ),
        ],
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.center,
      ),
    );
  }

  void _onAssetChange() {
    if (useAlbum) {
      _onPhotoRefresh();
    }
  }

  void _onPhotoRefresh() async {
    List<AssetPathEntity> pathList;
    switch (options.pickType) {
      case PickType.onlyImage:
        pathList = await PhotoManager.getAssetPathList(type: RequestType.image);
        break;
      case PickType.onlyVideo:
        pathList = await PhotoManager.getAssetPathList(type: RequestType.image);
        break;
      default:
        pathList = await PhotoManager.getAssetPathList();
    }

    if (pathList == null) {
      return;
    }

    this.galleryPathList.clear();
    this.galleryPathList.addAll(pathList);

    if (!this.galleryPathList.contains(assetProvider.current)) {
      // current path is deleted , 当前的相册被删除, 应该提示刷新
      if (this.galleryPathList.length > 0) {
        _onGalleryChange(this.galleryPathList[0]);
      }
      return;
    }
    // Not deleted
    _onGalleryChange(assetProvider.current);
  }
}
